SUBROUTINE FPDIAGFLASH(KST,KEND,KPROMA,KLEV,&
                     & PQL,PQI,PQR,PQS,PQGH,PDELP,PT,PVW,PLSM,PFLASH)
		     
USE PARKIND1 , ONLY : JPIM, JPRB
USE YOMHOOK  , ONLY : LHOOK, DR_HOOK
USE YOMFPC   , ONLY : NMTFLASH,RFLASHCAL,RFLASHQIL,RFLASHTMXPL,RFLASHTCRIT    
USE YOMCST   , ONLY : RG
USE YOMLUN   , ONLY : NULOUT

!******** FPDIAGFLASH  ************
!
!      PURPOSE:
!      --------
!      Compute flash rates (or lightening potential) 
!
!      INTERFACE:
!      ----------     
!      *CALL FPDIAGFLASH*
!      
!
!        EXPLICIT ARGUMENTS:
!        -------------------
!          INPUT:
!        KST     : start of work
!        KEND    : end of work
!        KPROMA  : dimension of work
!        KLEV    : number of levels
!        PT      : temperature (K)
!        PQL     : cloud water (g/kg)
!        PQI     : cloud ice (g/kg)
!        PQS     : snow (g/kg) 
!        PQR     : rain (g/kg)
!        PQGH    : graupel and hail (g/kg)
!        PWW     : vertical velocity  
!        PDELP   : layer thickness (Pa) 
!        PPFULL  : pressure on full levels (Pa)
!        PR      : gas constant (J/(kg K)
!        PVW     : vertical velocity (m/s)
!        PLSM    : land sea mask
!
!
!          OUTPUT:
!        PFLASH  : flash rate for given column
!
!        IMPLICIT ARGUMENTS:
!        -------------------
!           NONE
!
!      METHOD:
!      -------
!        different methods to compute flash rate can be chosen, dependent on  
!        NMTFLASH:
!        
!         = 0  Price and Rind (1992+)     :  flash rate estimate based on (concevtive) cloud top or max. vert. speed in conv. in column     
!         = 1  McCaul et. al  (2009) part 1: flash rate is based on upward flux of precipitatinf ice hydrometeors (graupel [and hail if available])
!               in the mixed phase region
!	  = 2  McCaul et. al  (2009) part 2: flash rate is based on vertically integrated amount of ice hydrometeors              
!         = 3  McCaul et. al  (2009) part 3: blending version 1 and 2
!
!      REFERENCE:
!      ----------
!      NMTFLASH=0  : Price, C. and Rind, D.: A simple lightning parameterization for calculating global lightning distributions, J. Geophys. Res., 97,
!                    1992, +1993, +1994
!      NMTFLASH=1-3: McCaul Jr, Eugene W., et al. "Forecasting lightning threat using cloud-resolving model simulations." 
!                    Weather and Forecasting 24.3 (2009): 709-729.McCaul et al.    
!
!      AUTHOR:
!      -------
!        Christoph Wittmann *ZAMG*
!
!      MODIFICATIONS:
!      --------------
!
!        original version: June 2013
!


USE PARKIND1, ONLY: JPIM, JPRB
USE YOMHOOK,  ONLY: LHOOK, DR_HOOK
USE YOMCST,   ONLY: RTT 



IMPLICIT NONE


INTEGER(KIND=JPIM),INTENT(IN) :: KST,KEND,KPROMA,KLEV
REAL(KIND=JPRB),INTENT(IN):: PQI(KPROMA,KLEV),PQL(KPROMA,KLEV),PQS(KPROMA,KLEV),PQR(KPROMA,KLEV)  
REAL(KIND=JPRB),INTENT(IN):: PQGH(KPROMA,KLEV),PDELP(KPROMA,KLEV)        
REAL(KIND=JPRB),INTENT(IN):: PVW(KPROMA,KLEV),PLSM(KPROMA),PT(KPROMA,KLEV)
REAL(KIND=JPRB),INTENT(OUT):: PFLASH(KPROMA)
! , PR(KPROMA,KLEV), PPFULL(KPROMA,KLEV) !

!local declarations
INTEGER (KIND=JPIM) :: JLON, JLEV
REAL(KIND=JPRB) :: ZHOOK_HANDLE
REAL (KIND=JPRB) :: ZINT(KPROMA), ZFLASH1(KPROMA), ZFLASH2(KPROMA),ZVW_MAX(KPROMA)
REAL (KIND=JPRB) :: ZTEST, ZTEST1, ZTEST2, ZTEST3, ZRESCAL, ZFINCAL
REAL (KIND=JPRB) :: ZRHOAIR, ZK1, ZK2, ZQILMIN, ZTMIX, ZVW
REAL (KIND=JPRB) :: ZPRFACZL,ZPRPOTZL,ZPRFACWL,ZPRPOTWL
REAL (KIND=JPRB) :: ZPRFACZS,ZPRPOTZS,ZPRFACWS,ZPRPOTWS

IF (LHOOK) CALL DR_HOOK('FPDIAGFLASH',0,ZHOOK_HANDLE)
! ------------------------------------------------

ZINT(:)=0.0_JPRB
ZQILMIN=RFLASHQIL      ! min value for qi+ql to detect "inside cloud" 1.E-7_JPRB
ZVW_MAX(:)=-100._JPRB  ! max wind speed (m/s)s

!used for Price and Rind:
!------------------------
! FAC .. factor or pot ... exponent;  Z or W ... based on cloud top z or vert. velocity W ;  L ... over land or S over sea
! over land:
ZPRFACZL=3.44E-5_JPRB   ! just used in the case we base our flashes on ztop
ZPRPOTZL=4.9_JPRB       ! just used in the case we base our flashes on ztop
ZPRFACWL=5.0E-6_JPRB
ZPRPOTWL=4.54_JPRB
!over seas (following modification from Michalon (1999)
ZPRFACZS=6.57E-4_JPRB   ! just used in the case we base our flashes on ztop
ZPRPOTZS=4.9_JPRB       ! just used in the case we base our flashes on ztop
ZPRFACWS=1.094E-6_JPRB    
ZPRPOTWS=4.495_JPRB    

!calibration factor (f(grid_size))
!division by 1.2E9 is to obtain SI units (unlike in the original paper)
!multiplication by 1.0E6 is to go to km2 from m2 and 
!multiplication by 300. to obtain flashes per 5 minutes
ZRESCAL=1.0_JPRB/1.2E9_JPRB*1.0E6_JPRB*300._JPRB

! used for McCaul:
!-----------------
ZTMIX=RTT-RFLASHTMXPL   
ZK1=0.042_JPRB 
ZK2=0.2_JPRB

! overall calibration:
ZFINCAL=RFLASHCAL

! 0. Method based on Price and Rind (1992+)
! ------------------------------------------------

IF (NMTFLASH == 0) THEN

  ! we have to find the maximum vertical velocity in a (convective) updraught
  
  DO JLEV=KLEV,1,-1
    DO JLON=KST,KEND
      
      ! we should verify that we are inside a cloud !!!!
      ZTEST1=MAX(0.0_JPRB,SIGN(1.0_JPRB,PQI(JLON,JLEV)+PQL(JLON,JLEV)-ZQILMIN)) 
      ! remember max vertical speed
      ZVW_MAX(JLON)=MAX(PVW(JLON,JLEV)*ZTEST1,ZVW_MAX(JLON))
 
    ENDDO
  ENDDO
  
  DO JLON=KST,KEND
   
    ZFLASH1(JLON)=PLSM(JLON)*(ZPRFACWL*ZVW_MAX(JLON)**ZPRPOTWL)+&
                  &(1._JPRB-PLSM(JLON))*(ZPRFACWS*ZVW_MAX(JLON)**ZPRPOTWS)
    ! normalization according to SI unit is missing (probably)
    ! no access to original paper!?
    ! calibration due to horizontal grid-size?
   	  
  ENDDO
  

ENDIF

IF (NMTFLASH == 1 .OR. NMTFLASH == 3) THEN

   ZINT(:)=0.0_JPRB
   
! first: find level closest to -15 degree isotherm inside a cloud
  DO JLEV=KST+2,KLEV,1
    DO JLON=KST,KEND
      
      ! test 1: inside a cloud? , test 2: temperture below -15 , test 3: temperatur above -15
      ! test= 1+2+3 should garantie that we've just passed the -15 degree isoterm inside a cloud
      ZTEST1=MAX(0.0_JPRB,SIGN(1.0_JPRB,PQI(JLON,JLEV)+PQL(JLON,JLEV)-ZQILMIN))         
      ZTEST2=MAX(0.0_JPRB,SIGN(1.0_JPRB,ZTMIX-PT(JLON,JLEV-1)))   
      ZTEST3=MAX(0.0_JPRB,SIGN(1.0_JPRB,PT(JLON,JLEV)-ZTMIX)) 
      ZTEST=ZTEST1*ZTEST2*ZTEST3
  
      ! if all test are successful we save the level (just below -15) inside a cloud we need 
      ! vertical velocity and graupe+hail content
      ! therefore we take avaerage from level below and above isoterm
      ! and multiply with vertical velocity (average from level below and above isotherm)
      ZVW=0.5_JPRB*(PVW(JLON,JLEV-1)+PVW(JLON,JLEV))
      !the following line should be active max 1 time during vertical loop
      ZINT(JLON)=ZINT(JLON)+ZTEST*0.5_JPRB*(PQGH(JLON,JLEV-1)+PQGH(JLON,JLEV))*ZVW
      
    ENDDO
  ENDDO

  DO JLON=KST,KEND
    ZFLASH1(JLON)=ZK1*ZRESCAL*ZINT(JLON)
  ENDDO

ENDIF  

IF (NMTFLASH == 2 .OR. NMTFLASH == 3) THEN

  ZINT(:)=0.0_JPRB

! we need to integrate solid hydrometeors inside a cloud
  DO JLEV=KLEV,1,-1
    DO JLON=KST, KEND
      ! if sum of cloud condensate exceeds a certain threshold we assume that we're inside a cloud
      ZTEST1=MAX(0.0_JPRB,SIGN(1.0_JPRB,PQI(JLON,JLEV)+PQL(JLON,JLEV)-ZQILMIN))
      ZINT(JLON)=ZINT(JLON)+ZTEST1*PDELP(JLON,JLEV)/RG*&
                 &(PQI(JLON,JLEV)+PQS(JLON,JLEV)+PQGH(JLON,JLEV))

!      below is the original definition of the parametrization by McCaul et al, 2009
!      it uses rho and geopotential thickness, 
!      at the moment, only hydrostatic approximation is fully coded and tested
!      for original implementation, changes to the dataflow are requiered!
!
!
!      ZRHOAIR=PPFULL(JLON,JLEV)/(PR(JLON,JLEV)*PT(JLON,JLEV))    
!      ZINT(JLON)=ZINT(JLON)+ZTEST1*ZRHOAIR* (PGEOPH(JLON,JLEV-1)-PGEOPH(JLON,JLEV))/RG*&
!                 &(PQI(JLON,JLEV)+PQS(JLON,JLEV)+PQGH(JLON,JLEV))

    ENDDO
  ENDDO
  
  DO JLON=KST,KEND
    ZFLASH2(JLON)=ZK2*ZRESCAL*ZINT(JLON)
  ENDDO

ENDIF  ! end of NTMFLASH

IF (NMTFLASH == 3 ) THEN
 
  ! McCaul version 3 is simply a weighted combination of version 1 and 2 (-> McCaul 2009)
  DO JLON=KST,KEND
    ZFLASH1(JLON)=0.95_JPRB*ZFLASH1(JLON)+0.05_JPRB*ZFLASH2(JLON)
  ENDDO
  

ENDIF

DO JLON=KST,KEND
!  PFLASH(JLON)=ZFLASH1(JLON)*ZFINCAL

  IF (NMTFLASH==2) THEN 
    PFLASH(JLON)=ZFLASH2(JLON)*ZFINCAL
  ELSE
    PFLASH(JLON)=ZFLASH1(JLON)*ZFINCAL
  ENDIF

ENDDO

IF (LHOOK) CALL DR_HOOK('FPDIAGFLASH',1,ZHOOK_HANDLE)


END SUBROUTINE FPDIAGFLASH 
