SUBROUTINE FPSHEAR(KST,KEND,KPROMA,KLEV,&
                     & PU,PV,PUCLS,PVCLS,POROG,PGEOPF,PDLS,PLLS)

USE PARKIND1 , ONLY : JPIM, JPRB
USE YOMHOOK  , ONLY : LHOOK, DR_HOOK
USE YOMCST   , ONLY : RG
USE YOMFPC   , ONLY : RDLSH, RLLSH


!******** FPSHEAR  ************

!      PURPOSE:
!      --------
!      Compute deep layer and low level shear (0-6km and 0-1km)

!      INTERFACE:
!      ----------     
!      *CALL FPSHEAR*
      

!        EXPLICIT ARGUMENTS:
!        -------------------
!          INPUT:
!        KST     : start of work
!        KEND    : end of work
!        KPROMA  : dimension of work
!        KLEV    : number of levels
!        PU      : u wind
!        PV      : v wind
!        PUCLS   : u wind at 10 m
!        PVCLS   : v wind at 10 m
!        POROG   : orography height
!        PGEOPF  : geopotential at full levels


!          OUTPUT:
!        PDLS : deep layer shear
!        PLLS : low level shear

!        IMPLICIT ARGUMENTS:
!        -------------------
!           NONE

!      METHOD:
!      -------
!        Simply computes differences for wind at the ground and at certain height

!      AUTHOR:
!      -------
!        Jure Cedilnik *ARSO*

!      MODIFICATIONS:
!      --------------

!        original version: August 2013
!



IMPLICIT NONE


INTEGER(KIND=JPIM),INTENT(IN) :: KST,KEND,KPROMA,KLEV
REAL(KIND=JPRB),INTENT(IN):: PU(KPROMA,KLEV),PV(KPROMA,KLEV),PUCLS(KPROMA),PVCLS(KPROMA) 
REAL(KIND=JPRB),INTENT(IN):: PGEOPF(KPROMA,KLEV), POROG(KPROMA)
REAL(KIND=JPRB),INTENT(OUT):: PDLS(KPROMA), PLLS(KPROMA)

!local declarations
INTEGER (KIND=JPIM) :: JLON, JLEV, IDLSLEV(KPROMA), ILLSLEV(KPROMA)
REAL(KIND=JPRB) :: ZHOOK_HANDLE



IF (LHOOK) CALL DR_HOOK('FPSHEAR',0,ZHOOK_HANDLE)
! ------------------------------------------------

PDLS(:)=0.0_JPRB
PLLS(:)=0.0_JPRB

! find levels at certain heights (DLSH and LLSH)
IDLSLEV(:)=KLEV
ILLSLEV(:)=KLEV

DO JLEV=KLEV,1,-1
  DO JLON=KST, KEND
    IDLSLEV(JLON)= IDLSLEV(JLON) - &
   &   MAX(0.0_JPRB,SIGN(1.0_JPRB,RDLSH*RG+POROG(JLON)-PGEOPF(JLON,JLEV)))
    ILLSLEV(JLON)= ILLSLEV(JLON) - &
   &   MAX(0.0_JPRB,SIGN(1.0_JPRB,RLLSH*RG+POROG(JLON)-PGEOPF(JLON,JLEV)))
  ENDDO
ENDDO
  
! compute shear (vector norm of wind difference at two heights)
DO JLON=KST,KEND
   PDLS(JLON)=((PU(JLON,IDLSLEV(JLON))-PUCLS(JLON))**2+(PV(JLON,IDLSLEV(JLON))-PVCLS(JLON))**2)**0.5_JPRB
   PLLS(JLON)=((PU(JLON,ILLSLEV(JLON))-PUCLS(JLON))**2+(PV(JLON,ILLSLEV(JLON))-PVCLS(JLON))**2)**0.5_JPRB
!   PDLS(JLON)=ILLSLEV(JLON)
!   PLLS(JLON)=IDLSLEV(JLON)
ENDDO

IF (LHOOK) CALL DR_HOOK('FPSHEAR',1,ZHOOK_HANDLE)

END SUBROUTINE FPSHEAR
